@@ -9,14 +9,22 @@
  *
  */
 
-#ifndef CLIENTS_EXECUTE_H
-#define CLIENTS_EXECUTE_H
+#pragma once 
+
+#include <stdio.h>
+
 #include "libmemcached/memcached.h"
-#include "generator.h"
+#include "clients/generator.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 unsigned int execute_set(memcached_st *memc, pairs_st *pairs, unsigned int number_of);
 unsigned int execute_get(memcached_st *memc, pairs_st *pairs, unsigned int number_of);
 unsigned int execute_mget(memcached_st *memc, const char * const *keys, size_t *key_length,
                           unsigned int number_of);
-#endif
 
+#ifdef __cplusplus
+} // extern "C"
+#endif
