@@ -39,6 +39,7 @@
 
 /* These are private */ 
 #define memcached_is_allocated(__object) ((__object)->options.is_allocated)
+#define memcached_is_encrypted(__object) ((__object)->hashkit._key)
 #define memcached_is_udp(__object) ((__object)->flags.use_udp)
 #define memcached_is_verify_key(__object) ((__object)->flags.verify_key)
 #define memcached_is_binary(__object) ((__object)->flags.binary_protocol)
@@ -51,7 +52,8 @@
 
 #define memcached_has_error(__object) ((__object)->error_messages)
 
-#define memcached_set_purging(__object, __value) ((__object)->state.is_purging= (__value))
+#define memcached_has_replicas(__object) ((__object)->root->number_of_replicas)
+
 #define memcached_set_processing_input(__object, __value) ((__object)->state.is_processing_input= (__value))
 #define memcached_set_initialized(__object, __value) ((__object)->options.is_initialized(= (__value))
 #define memcached_set_allocated(__object, __value) ((__object)->options.is_allocated= (__value))
