@@ -124,7 +124,7 @@ static inline bool _memcached_init(memcached_st *self)
   return true;
 }
 
-static void _free(memcached_st *ptr, bool release_st)
+static void __memcached_free(memcached_st *ptr, bool release_st)
 {
   /* If we have anything open, lets close it now */
   send_quit(ptr);
@@ -158,7 +158,9 @@ static void _free(memcached_st *ptr, bool release_st)
     ptr->configure.filename= NULL;
   }
 
-  if (memcached_is_allocated(ptr) && release_st)
+  hashkit_free(&ptr->hashkit);
+
+  if (memcached_is_allocated(ptr) and release_st)
   {
     libmemcached_free(ptr, ptr);
   }
@@ -172,7 +174,7 @@ memcached_st *memcached_create(memcached_st *ptr)
   }
   else
   {
-    ptr= (memcached_st *)malloc(sizeof(memcached_st));
+    ptr= (memcached_st *)libmemcached_xmalloc(NULL, memcached_st);
 
     if (ptr == NULL)
     {
@@ -182,11 +184,6 @@ memcached_st *memcached_create(memcached_st *ptr)
     ptr->options.is_allocated= true;
   }
 
-#if 0
-  memcached_set_purging(ptr, false);
-  memcached_set_processing_input(ptr, false);
-#endif
-
   if (_memcached_init(ptr) == false)
   {
     memcached_free(ptr);
@@ -206,30 +203,49 @@ memcached_st *memcached_create(memcached_st *ptr)
 
 memcached_st *memcached(const char *string, size_t length)
 {
-  memcached_st *self= memcached_create(NULL);
-  if (self == NULL)
+  if (length == 0 and string)
+  {
+    return NULL;
+  }
+
+  if (length and string == NULL)
   {
     return NULL;
   }
 
   if (length == 0)
   {
-    return self;
+    if (bool(getenv("LIBMEMCACHED")))
+    {
+      string= getenv("LIBMEMCACHED");
+      length= string ? strlen(string) : 0;
+    }
   }
 
-  memcached_return_t rc= memcached_parse_configuration(self, string, length);
-  if (memcached_success(rc) and memcached_parse_filename(self))
+  memcached_st *memc= memcached_create(NULL);
+  if (memc == NULL)
   {
-    rc= memcached_parse_configure_file(*self, memcached_parse_filename(self), memcached_parse_filename_length(self));
+    return NULL;
+  }
+
+  if (length == 0 or string == NULL)
+  {
+    return memc;
+  }
+
+  memcached_return_t rc= memcached_parse_configuration(memc, string, length);
+  if (memcached_success(rc) and memcached_parse_filename(memc))
+  {
+    rc= memcached_parse_configure_file(*memc, memcached_parse_filename(memc), memcached_parse_filename_length(memc));
   }
     
   if (memcached_failed(rc))
   {
-    memcached_free(self);
+    memcached_free(memc);
     return NULL;
   }
 
-  return self;
+  return memc;
 }
 
 memcached_return_t memcached_reset(memcached_st *ptr)
@@ -242,7 +258,7 @@ memcached_return_t memcached_reset(memcached_st *ptr)
 
   bool stored_is_allocated= memcached_is_allocated(ptr);
   uint64_t query_id= ptr->query_id;
-  _free(ptr, false);
+  __memcached_free(ptr, false);
   memcached_create(ptr);
   memcached_set_allocated(ptr, stored_is_allocated);
   ptr->query_id= query_id;
@@ -281,7 +297,7 @@ void memcached_free(memcached_st *ptr)
 {
   if (ptr)
   {
-    _free(ptr, true);
+    __memcached_free(ptr, true);
   }
 }
 
@@ -354,6 +370,7 @@ memcached_st *memcached_clone(memcached_st *clone, const memcached_st *source)
 
   new_clone->_namespace= memcached_array_clone(new_clone, source->_namespace);
   new_clone->configure.filename= memcached_array_clone(new_clone, source->_namespace);
+  new_clone->configure.version= source->configure.version;
 
   if (LIBMEMCACHED_WITH_SASL_SUPPORT and source->sasl.callbacks)
   {
@@ -381,11 +398,21 @@ memcached_st *memcached_clone(memcached_st *clone, const memcached_st *source)
 
 void *memcached_get_user_data(const memcached_st *ptr)
 {
+  if (ptr == NULL)
+  {
+    return NULL;
+  }
+
   return ptr->user_data;
 }
 
 void *memcached_set_user_data(memcached_st *ptr, void *data)
 {
+  if (ptr == NULL)
+  {
+    return NULL;
+  }
+
   void *ret= ptr->user_data;
   ptr->user_data= data;
 
@@ -399,18 +426,30 @@ memcached_return_t memcached_push(memcached_st *destination, const memcached_st
 
 memcached_server_write_instance_st memcached_server_instance_fetch(memcached_st *ptr, uint32_t server_key)
 {
+  if (ptr == NULL)
+  {
+    return NULL;
+  }
+
   return &ptr->servers[server_key];
 }
 
 memcached_server_instance_st memcached_server_instance_by_position(const memcached_st *ptr, uint32_t server_key)
 {
+  if (ptr == NULL)
+  {
+    return NULL;
+  }
+
   return &ptr->servers[server_key];
 }
 
 uint64_t memcached_query_id(const memcached_st *self)
 {
-  if (not self)
+  if (self == NULL)
+  {
     return 0;
+  }
 
   return self->query_id;
 }
