@@ -36,8 +36,18 @@
 
 #pragma once
 
+#include <config.h>
+
 #include <libmemcached-1.0/struct/memcached.h>
 
+#ifdef __cplusplus
+#include <cstddef>
+#include <cstdlib>
+#else
+#include <stddef.h>
+#include <stdlib.h>
+#endif
+
 static inline void libmemcached_free(const memcached_st *self, void *mem)
 {
   if (self)
@@ -46,7 +56,11 @@ static inline void libmemcached_free(const memcached_st *self, void *mem)
   }
   else if (mem)
   {
+#ifdef __cplusplus
+    std::free(mem);
+#else
     free(mem);
+#endif
   }
 }
 
@@ -57,7 +71,11 @@ static inline void *libmemcached_malloc(const memcached_st *self, const size_t s
     return self->allocators.malloc(self, size, self->allocators.context);
   }
 
+#ifdef __cplusplus
+  return std::malloc(size);
+#else
   return malloc(size);
+#endif
 }
 #define libmemcached_xmalloc(__memcachd_st, __type) ((__type *)libmemcached_malloc((__memcachd_st), sizeof(__type)))
 
@@ -68,7 +86,11 @@ static inline void *libmemcached_realloc(const memcached_st *self, void *mem, si
     return self->allocators.realloc(self, mem, nmemb * size, self->allocators.context);
   }
 
-  return realloc(mem, size);
+#ifdef __cplusplus
+    return std::realloc(mem, size);
+#else
+    return realloc(mem, size);
+#endif
 }
 #define libmemcached_xrealloc(__memcachd_st, __mem, __nelem, __type) ((__type *)libmemcached_realloc((__memcachd_st), (__mem), (__nelem), sizeof(__type)))
 #define libmemcached_xvalloc(__memcachd_st, __nelem, __type) ((__type *)libmemcached_realloc((__memcachd_st), NULL, (__nelem), sizeof(__type)))
@@ -80,6 +102,10 @@ static inline void *libmemcached_calloc(const memcached_st *self, size_t nelem,
     return self->allocators.calloc(self, nelem, size, self->allocators.context);
   }
 
-  return calloc(nelem, size);
+#ifdef __cplusplus
+    return std::calloc(nelem, size);
+#else
+    return calloc(nelem, size);
+#endif
 }
 #define libmemcached_xcalloc(__memcachd_st, __nelem, __type) ((__type *)libmemcached_calloc((__memcachd_st), (__nelem), sizeof(__type)))
