@@ -39,10 +39,48 @@
 
 #include <libmemcached/common.h>
 
+#define memcached_set_purging(__object, __value) ((__object)->state.is_purging= (__value))
 
-memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
+class Purge
+{
+public:
+  Purge(memcached_st* arg) :
+    _memc(arg)
+  {
+    memcached_set_purging(_memc, true);
+  }
+
+  ~Purge()
+  {
+    memcached_set_purging(_memc, false);
+  }
+
+private:
+  memcached_st* _memc;
+};
+
+class PollTimeout
+{
+public:
+  PollTimeout(memcached_st* arg) :
+    _timeout(arg->poll_timeout),
+    _origin(arg->poll_timeout)
+  {
+    _origin = 2000;
+  }
+
+  ~PollTimeout()
+  {
+    _origin= _timeout;
+  }
+
+private:
+  int32_t _timeout;
+  int32_t& _origin;
+};
+
+bool memcached_purge(memcached_server_write_instance_st ptr)
 {
-  memcached_return_t ret= MEMCACHED_SUCCESS;
   memcached_st *root= (memcached_st *)ptr->root;
 
   if (memcached_is_purging(ptr->root) || /* already purging */
@@ -51,14 +89,14 @@ memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
       (ptr->io_bytes_sent >= ptr->root->io_bytes_watermark &&
        memcached_server_response_count(ptr) < 2))
   {
-    return MEMCACHED_SUCCESS;
+    return true;
   }
 
   /*
     memcached_io_write and memcached_response may call memcached_purge
     so we need to be able stop any recursion.. 
   */
-  memcached_set_purging(root, true);
+  Purge set_purge(root);
 
   WATCHPOINT_ASSERT(ptr->fd != INVALID_SOCKET);
   /* 
@@ -67,28 +105,26 @@ memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
   */
   if (memcached_io_write(ptr) == false)
   {
-    memcached_set_purging(root, true);
-
-    return memcached_set_error(*ptr, MEMCACHED_WRITE_FAILURE, MEMCACHED_AT);
+    memcached_set_error(*ptr, MEMCACHED_WRITE_FAILURE, MEMCACHED_AT);
+    return false;
   }
   WATCHPOINT_ASSERT(ptr->fd != INVALID_SOCKET);
 
+  bool is_successful= true;
   uint32_t no_msg= memcached_server_response_count(ptr) - 1;
   if (no_msg > 0)
   {
     memcached_result_st result;
-    memcached_result_st *result_ptr;
 
     /*
      * We need to increase the timeout, because we might be waiting for
      * data to be sent from the server (the commands was in the output buffer
      * and just flushed
    */
-    const int32_t timeo= ptr->root->poll_timeout;
-    root->poll_timeout= 2000;
+    PollTimeout poll_timeout(ptr->root);
 
-    result_ptr= memcached_result_create(root, &result);
-    WATCHPOINT_ASSERT(result_ptr);
+    memcached_result_st* result_ptr= memcached_result_create(root, &result);
+    assert(result_ptr);
 
     for (uint32_t x= 0; x < no_msg; x++)
     {
@@ -103,10 +139,7 @@ memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
       {
         WATCHPOINT_ERROR(rc);
         memcached_io_reset(ptr);
-        ret= rc;
-#if 0
-        ret= memcached_set_error(*ptr, rc, MEMCACHED_AT);
-#endif
+        is_successful= false;
       }
 
       if (ptr->root->callbacks != NULL)
@@ -116,8 +149,7 @@ memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
         {
           for (uint32_t y= 0; y < cb.number_of_callback; y++)
           {
-            rc = (*cb.callback[y])(ptr->root, result_ptr, cb.context);
-            if (rc != MEMCACHED_SUCCESS)
+            if (memcached_fatal((*cb.callback[y])(ptr->root, result_ptr, cb.context)))
             {
               break;
             }
@@ -127,9 +159,7 @@ memcached_return_t memcached_purge(memcached_server_write_instance_st ptr)
     }
 
     memcached_result_free(result_ptr);
-    root->poll_timeout= timeo;
   }
-  memcached_set_purging(root, false);
 
-  return ret;
+  return is_successful;
 }
