@@ -43,12 +43,11 @@ using namespace libtest;
 #include <vector>
 #include <string>
 #include <cerrno>
-#include <cassert>
 
 #include <libmemcached/memcached.h>
 #include <libmemcached/util.h>
 
-#include <tests/parser.h>
+#include <tests/libmemcached-1.0/parser.h>
 #include <tests/print.h>
 
 enum scanner_type_t
@@ -621,26 +620,25 @@ static memcached_return_t dump_server_information(const memcached_st *,
 {
   if (strcmp(memcached_server_name(instance), "localhost")) 
   {
-    assert(not memcached_server_name(instance));
+    fatal_assert(not memcached_server_name(instance));
     return MEMCACHED_FAILURE;
   }
 
   if (memcached_server_port(instance) < 8888 or memcached_server_port(instance) > 8892)
   {
-    assert(not memcached_server_port(instance));
+    fatal_assert(not memcached_server_port(instance));
     return MEMCACHED_FAILURE;
   }
 
   if (instance->weight > 5 or instance->weight < 2)
   {
-    assert(not instance->weight);
+    fatal_assert(not instance->weight);
     return MEMCACHED_FAILURE;
   }
 
   return MEMCACHED_SUCCESS;
 }
 
-
 test_return_t test_hostname_port_weight(memcached_st *)
 {
   const char *server_string= "--server=localhost:8888/?2 --server=localhost:8889/?3 --server=localhost:8890/?4 --server=localhost:8891/?5 --server=localhost:8892/?3";
